extends Control

const save_file = "user://data.tres"

var sound = true
var vibrate = true


func _ready():
	var data = loadStats()
	sound = data.sound
	vibrate = data.vibrate
	Singleton.lang = getLang(data.language)
	updateText()


func _draw():
	draw_line(Vector2(-150, -150), Vector2(150, -150), Color.BLACK, 140)
	draw_circle(Vector2(-150, -150), 70, Color.BLACK)
	draw_circle(Vector2(150, -150), 70, Color.BLACK)
	
	if sound:
		draw_line(Vector2(-150, -150), Vector2(150, -150), Color.WHITE, 120)
		draw_circle(Vector2(-150, -150), 60, Color.WHITE)
		draw_circle(Vector2(150, -150), 60, Color.WHITE)
	else:
		draw_line(Vector2(-150, -150), Vector2(150, -150), Color.DARK_GRAY, 120)
		draw_circle(Vector2(-150, -150), 60, Color.DARK_GRAY)
		draw_circle(Vector2(150, -150), 60, Color.DARK_GRAY)

	draw_line(Vector2(-150, 150), Vector2(150, 150), Color.BLACK, 140)
	draw_circle(Vector2(-150, 150), 70, Color.BLACK)
	draw_circle(Vector2(150, 150), 70, Color.BLACK)
	
	if vibrate:
		draw_line(Vector2(-150, 150), Vector2(150, 150), Color.WHITE, 120)
		draw_circle(Vector2(-150, 150), 60, Color.WHITE)
		draw_circle(Vector2(150, 150), 60, Color.WHITE)
	else:
		draw_line(Vector2(-150, 150), Vector2(150, 150), Color.DARK_GRAY, 120)
		draw_circle(Vector2(-150, 150), 60, Color.DARK_GRAY)
		draw_circle(Vector2(150, 150), 60, Color.DARK_GRAY)


func _on_config_toggle_sound():
	if sound:
		sound = false
		$Sounds/RichTextLabel.text = tr("SOUND_OFF")
	else:
		sound = true
		$Sounds/RichTextLabel.text = tr("SOUND")
	saveStats()
	queue_redraw()


func _on_config_toggle_vibrate():
	if vibrate:
		vibrate = false
		$Vibrate/RichTextLabel.text = tr("VIBRATION_OFF")
	else:
		vibrate = true
		$Vibrate/RichTextLabel.text = tr("VIBRATION")
	saveStats()
	queue_redraw()


func saveStats():
	var data = loadStats()
	data.sound = sound
	data.vibrate = vibrate
	data.language = enumerateLang(Singleton.lang)
	ResourceSaver.save(data, save_file)


func enumerateLang(lang):
	match lang:
		"en":
			return 0
		"de":
			return 1
		"esp":
			return 2


# For when I add new things to the gamedata
func updateStatsFile(data):
	if "version" in data and data.version == Singleton.dataVersion:
		return data
	else:
		var newData = GameData.new()
		newData.playedGames = data.playedGames
		newData.wonGames = data.wonGames
		newData.totalMines = data.totalMines
		
		newData.fastestEasy = data.fastestEasy
		newData.fastestEasyMines = data.fastestEasyMines
		newData.wonEasy = data.wonEasy
		newData.totalEasy = data.totalEasy

		newData.fastestMediumMines = data.fastestMediumMines
		newData.fastestMedium = data.fastestMedium
		newData.wonMedium = data.wonMedium
		newData.totalMedium = data.totalMedium
		
		newData.fastestHardMines = data.fastestHardMines
		newData.fastestHard = data.fastestHard
		newData.wonHard = data.wonHard
		newData.totalHard = data.totalHard
		
		newData.sound = data.sound
		newData.vibrate = data.vibrate

		if "language" in data:
			newData.language = data.language

		ResourceSaver.save(data, save_file)
		return newData


func loadStats():
	if ResourceLoader.exists(save_file):
		return updateStatsFile(load(save_file))
	return GameData.new()


func updateText():
	if sound:
		$Sounds/RichTextLabel.text =  tr("SOUND")
	else:
		$Sounds/RichTextLabel.text =  tr("SOUND_OFF")

	if vibrate:
		$Vibrate/RichTextLabel.text =  tr("VIBRATION")
	else:
		$Vibrate/RichTextLabel.text =  tr("VIBRATION_OFF")


func _on_config_reset_text():
	updateText()
	saveStats()


func getLang(langNum):
	match langNum:
		0:
			return "en"
		1:
			return "de"
		2:
			return "esp"
